
#include <stdio.h>
#include <allegro5/allegro.h>
#include <allegro5/allegro_opengl.h>
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_image.h>
#include <allegro5/allegro_native_dialog.h>
#include "spheroid.h"
#include "mersenne.h"

#define SCRW 800
#define SCRH 600

double start;

int display_width;
int display_height;
bool key_array[ALLEGRO_KEY_MAX];

GLfloat   sun_position[] = { 10.0, 10.0,  -10.0, 0.0 };

GLfloat sphere_ambient[]   = { 0.1, 0.1, 0.1, 1.0 };
GLfloat sphere_diffuse[]   = { 0.9, 0.9, 0.9, 1.0 };
GLfloat sphere_specular[]  = { 0.3, 0.3, 0.3, 1.0 };
GLfloat sphere_emissive[]  = { 0.0, 0.0, 0.0, 1.0 };

POINT3D position = { -1201.0, -360.0, -1725.0 };
POINT3D vector = { 0.0, -8.0, 0.0 };

POINT3D farm_location = { 1200.0, 331.0, 1700.0 };

float playerheight = 4.0;

int faceimgID;
int terrainimgID;
int skyimgID;
int wallimgID;
int roofimgID;
int windowimgID;
int doorimgID;

extern int terrainlist;
extern float *terrainfloats;
extern int maketerrain(void);
extern void drawsky(void);
extern void draw_cabin(void);

char distmessages[128][128] =
{
 "Here be dragons!",
 "There's bandits over there!",
 "Your girlfriend misses you!",
 "The crops need tending!",
 "You'll fall off the edge of the world!",
 "Your curiosity abandons you"
};

char distmessage2[] = "Get back to your farm!";

int main(int argc, char **argv)
{
	ALLEGRO_EVENT_QUEUE *queue;
	ALLEGRO_TIMER *timer;
	ALLEGRO_DISPLAY *display;
	ALLEGRO_PATH *path;
	ALLEGRO_BITMAP *faceimg;
	ALLEGRO_BITMAP *terrainimg;
	ALLEGRO_BITMAP *skyimg;
	ALLEGRO_BITMAP *wallimg;
	ALLEGRO_BITMAP *roofimg;
	ALLEGRO_BITMAP *windowimg;
	ALLEGRO_BITMAP *doorimg;

	ALLEGRO_FONT *font;
	ALLEGRO_COLOR red;
	ALLEGRO_COLOR tan;
	ALLEGRO_COLOR brown;
	ALLEGRO_EVENT event;
	float movespeed;
	float playerdist;	//distance from farm
	int msg = 0;
	int distwarn = 0;

	int frames = 0;
	int redraw = 0;

	if(!al_init())
	{
		al_show_native_message_box(0, "Error", "Failed to initialize Allegro", "", NULL, 0);
		return 1;
	}

	//If user runs it from somewhere else...
	path = al_get_standard_path(ALLEGRO_RESOURCES_PATH);
	al_change_directory(al_path_cstr(path,ALLEGRO_NATIVE_PATH_SEP));

	al_install_keyboard();
	al_install_mouse();
	al_init_font_addon();
	al_init_ttf_addon();
	al_init_image_addon();
	al_init_primitives_addon();

	al_set_new_display_option(ALLEGRO_SAMPLE_BUFFERS, 1, ALLEGRO_SUGGEST);
	al_set_new_display_option(ALLEGRO_SAMPLES, 4, ALLEGRO_SUGGEST);
	al_set_new_display_option(ALLEGRO_VSYNC, 1, ALLEGRO_SUGGEST);
	al_set_new_display_option(ALLEGRO_DEPTH_SIZE,24,ALLEGRO_REQUIRE);
	//al_set_new_display_option(ALLEGRO_FLOAT_DEPTH
	al_set_new_display_flags(ALLEGRO_OPENGL | ALLEGRO_NOFRAME);
	display_width = SCRW;
	display_height = SCRH;
	display = al_create_display(display_width, display_height);
	if (!display)
	{
		al_show_native_message_box(0, "Error", "Couldn't create display", "", NULL, 0);
		return 1;
	}

	red = al_map_rgba(255,0,0,255);
	tan = al_map_rgba(255, 249, 175, 255);
	brown = al_map_rgba(129,91,46,255);

	font = al_load_font("luxisr.ttf", -20, 0);
	if(!font)
	{
		al_show_native_message_box(0, "Error", "Unable to load \"luxisr.ttf\"", "", NULL, 0);
		return 1;
	}

	faceimg = al_load_bitmap("face.tga");
	if(!faceimg)
	{
		al_show_native_message_box(0, "Error", "Unable to load \"face.tga\"", "", NULL, 0);
		return 0;
	}

	terrainimg = al_load_bitmap("terrain.jpg");
	if(!terrainimg)
	{
		al_show_native_message_box(0,"Error","Unable to load \"terrain.jpg\"","",NULL,0);
		return 0;
	}

	skyimg = al_load_bitmap("sky.tga");
	if(!skyimg)
	{
		al_show_native_message_box(0, "Error", "Unable to load \"sky.tga\"", "", NULL, 0);
		return 0;
	}

	wallimg = al_load_bitmap("wall.jpg");
	if(!skyimg)
	{
		al_show_native_message_box(0, "Error", "Unable to load \"wall.jpg\"", "", NULL, 0);
		return 0;
	}

	roofimg = al_load_bitmap("roof.jpg");
	if(!skyimg)
	{
		al_show_native_message_box(0, "Error", "Unable to load \"roof.jpg\"", "", NULL, 0);
		return 0;
	}

	windowimg = al_load_bitmap("window.jpg");
	if(!skyimg)
	{
		al_show_native_message_box(0, "Error", "Unable to load \"window.jpg\"", "", NULL, 0);
		return 0;
	}

	doorimg = al_load_bitmap("door.jpg");
	if(!skyimg)
	{
		al_show_native_message_box(0, "Error", "Unable to load \"door.jpg\"", "", NULL, 0);
		return 0;
	}

	faceimgID = al_get_opengl_texture(faceimg);
	glBindTexture(GL_TEXTURE_2D,faceimgID);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	terrainimgID = al_get_opengl_texture(terrainimg);
	glBindTexture(GL_TEXTURE_2D,terrainimgID);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	skyimgID = al_get_opengl_texture(skyimg);
	glBindTexture(GL_TEXTURE_2D,skyimgID);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	wallimgID = al_get_opengl_texture(wallimg);
	glBindTexture(GL_TEXTURE_2D,wallimgID);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	roofimgID = al_get_opengl_texture(roofimg);
	glBindTexture(GL_TEXTURE_2D,roofimgID);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	windowimgID = al_get_opengl_texture(windowimg);
	glBindTexture(GL_TEXTURE_2D,windowimgID);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	doorimgID = al_get_opengl_texture(doorimg);
	glBindTexture(GL_TEXTURE_2D,doorimgID);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);


	glEnable(GL_BLEND);
	glEnable(GL_ALPHA_TEST);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);


	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glLightfv(GL_LIGHT0, GL_POSITION, sun_position);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	/*
	glLightfv(GL_LIGHT0, GL_AMBIENT,sphere_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,sphere_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR,sphere_specular);
	glLightfv(GL_LIGHT0, GL_EMISSION,sphere_emissive);
	*/
	//glClearColor(0.5, 0.5, 1.0, 1.0);  //background will be black

	//glLightModelfv(GL_LIGHT_MODEL_AMBIENT,ambientLight);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,sphere_ambient);
	//glLightModelfv(GL_LIGHT_MODEL_DIFFUSE,sphere_diffuse);
	//glLightModelfv(GL_LIGHT_MODEL_SPECULAR,sphere_specular);
	//glLightModelfv(GL_LIGHT_MODEL_EMISSION,sphere_emissive);

	glMaterialfv(GL_FRONT, GL_SPECULAR, sphere_specular);
	glMaterialf (GL_FRONT, GL_SHININESS, 14.0);
	glMaterialfv(GL_FRONT, GL_AMBIENT, sphere_ambient);
	//glMaterialfv(GL_FRONT, GL_EMISSION,sphere_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,sphere_diffuse);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);

	timer = al_create_timer(1.0 / 60.0);

	//al_set_window_position(display,320,240);
	queue = al_create_event_queue();
	al_register_event_source(queue, al_get_keyboard_event_source());
	al_register_event_source(queue, al_get_display_event_source(display));
	al_register_event_source(queue, al_get_mouse_event_source());
	al_register_event_source(queue, al_get_timer_event_source(timer));

	genpts();

	if(maketerrain())
	{
		al_show_native_message_box(0, "Error", "Out of memory generating terrain", "", NULL, 0);
		return 1;
	}

	start = al_get_time();

	al_start_timer(timer);

	while (1) //frames < 1)
	{
		al_wait_for_event(queue, &event);
		{
			switch (event.type)
			{
				case ALLEGRO_EVENT_DISPLAY_CLOSE:
					goto done;

				case ALLEGRO_EVENT_KEY_CHAR:
				{
					int k;
					k = event.keyboard.keycode;

					if (k == ALLEGRO_KEY_ESCAPE)
					{
						goto done;
					}
				}

				case ALLEGRO_EVENT_KEY_DOWN:
				{
					key_array[event.keyboard.keycode] = true;
					break;
				}

				case ALLEGRO_EVENT_KEY_UP:
					key_array[event.keyboard.keycode] = false;
					break;


				case ALLEGRO_EVENT_TIMER:
				{
					redraw++;
				}
			}

			while( (redraw > 0) && al_is_event_queue_empty(queue))
			{
				glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

				glMatrixMode(GL_PROJECTION);
				glLoadIdentity();
				glFrustum(-1.0, 1.0, -(6.0/8.0), (6.0/8.0), 1.0, 10000.0);

				glMatrixMode(GL_MODELVIEW);

				/*	showing the smiley round face here
				glLoadIdentity();		//reset modelview matrix to avoid accumulating rotations and translations
				glTranslatef(0.0, 0.0, -4.0);	//left side of screen, push back so we can see it on other side of near frustum plane
				glRotatef(rot2,1.0,0.0,0.0);
				glRotatef(rot,0.0,1.0,0.0);
				rot += 0.38;
				if(rot > 360.0)
					rot -= 360.0;
				rot2 += 0.051;
				if(rot2 > 360.0)
					rot2 -= 360.0;

				if(frames & 0x80)
					glColor3f(1.0,0.0,0.0);
				else
					glColor3f(0.0,0.0,1.0);
				glRotatef(90.0,1.0,0.0,0.0);
				glRotatef(90.0,0.0,0.0,1.0);

				glBindTexture(GL_TEXTURE_2D,faceimgID);
				glCallList(unitsphere);
				*/
				glColor4f(1.0,1.0,1.0,1.0);
				glDisable(GL_LIGHTING);
				drawsky();
				glBindTexture(GL_TEXTURE_2D,terrainimgID);
				glLoadIdentity();
				glRotatef(vector.x,1.0,0.0,0.0);
				glRotatef(vector.y,0.0,1.0,0.0);
				glRotatef(vector.z,0.0,0.0,1.0);
				glTranslatef(position.x,position.y,position.z);

				/*
				 * left/right arrow keys rotate view
				 * up/down arrow keys move forward/backward
				 * either alt makes left/right strafe
				 * pitch is page up/down
				 */

				movespeed = 0.2;

				if(key_array[ALLEGRO_KEY_LSHIFT] || key_array[ALLEGRO_KEY_RSHIFT])
				{
					movespeed = 0.7;
				}

				if(key_array[ALLEGRO_KEY_PGUP])
				{
					vector.x += 1.0;
					if(vector.x > 360.0)
					{
						vector.x -= 360.0;
					}
				}

				if(key_array[ALLEGRO_KEY_PGDN])
				{
					vector.x -= 1.0;
					if(vector.x < 0.0)
					{
						vector.x += 360.0;
					}
				}

				if(key_array[ALLEGRO_KEY_ALT])
				{
					if(key_array[ALLEGRO_KEY_LEFT])
					{
						position.x += cos(DEGREES_TO_RADIANS(vector.y)) * movespeed;
						position.z += sin(DEGREES_TO_RADIANS(vector.y)) * movespeed;
					}

					if(key_array[ALLEGRO_KEY_RIGHT])
					{
						position.x -= cos(DEGREES_TO_RADIANS(vector.y)) * movespeed;
						position.z -= sin(DEGREES_TO_RADIANS(vector.y)) * movespeed;
					}
				}
				else
				{
					if(key_array[ALLEGRO_KEY_LEFT])
					{
						vector.y -= 1.0;
						if(vector.y < 0.0)
						{
							vector.y += 360.0;
						}
					}
					if(key_array[ALLEGRO_KEY_RIGHT])
					{
						vector.y += 1.0;
						if(vector.y > 360.0)
						{
							vector.y -= 360.0;
						}
					}
				}

				if(key_array[ALLEGRO_KEY_UP])
				{
					position.x -= sin(DEGREES_TO_RADIANS(vector.y)) * movespeed;
					position.z += cos(DEGREES_TO_RADIANS(vector.y)) * movespeed;
				}

				if(key_array[ALLEGRO_KEY_DOWN])
				{
					position.x += sin(DEGREES_TO_RADIANS(vector.y)) * movespeed;
					position.z -= cos(DEGREES_TO_RADIANS(vector.y)) * movespeed;
				}

				position.y = get_ground_height(&position) - playerheight;

				//printf("x % 2.4f y % 2.4f z % 2.4f xa % 2.4f ya % 2.4f za % 2.4f\n",position.x,position.y,position.z,vector.x,vector.y,vector.z);
				glCallList(terrainlist);

				//glTranslatef(1280.0,280.0,1028.0);
				glTranslatef(farm_location.x, farm_location.y, farm_location.z);
				draw_cabin();

				glMatrixMode(GL_TEXTURE);
				glPushMatrix();
				glLoadIdentity();
				glMatrixMode(GL_PROJECTION);
				glPushMatrix();
				glLoadIdentity();
				glMatrixMode(GL_MODELVIEW);
				glPushMatrix();
				glLoadIdentity();
				glDisable(GL_TEXTURE_2D);
				glDisable(GL_CULL_FACE);

				glMatrixMode(GL_PROJECTION);
				glOrtho(0.0, display_width, display_height, 0.0, -1.0, 1000.0);

				playerdist = sqrt( (position.x + farm_location.x) * (position.x + farm_location.x) + (position.z + farm_location.z) * (position.z + farm_location.z));

				//al_draw_textf(font,red,20.0,20.0,0,"% 2.4f % 2.4f % 2.4f % 2.4f % 2.4f % 2.4f % 2.4f",position.x,position.y,position.z,vector.x,vector.y,vector.z,playerdist);

				if(playerdist > 1000.0)
				{
					//i'm to rushed to look up 2d dot product atm.
					/*
					float extradist;
					float totdistx = (position.x + vector.x + farm_location.x) * 2.0;
					float totdistz = (position.z + vector.z + farm_location.z) * 2.0;
					extradist = sqrt(totdistx * totdistz);
					if(extradist > playerdist)
					{
						*/
						if(distwarn == 0)
						{
							msg = genrand_int32() % 6;
						}
						distwarn = 1;
					//}
				}
				else
				{
					distwarn = 0;
				}

				if(distwarn)
				{
					glDisable(GL_DEPTH_TEST);
					al_draw_filled_rounded_rectangle( SCRW/2 - 200,  SCRH/2 - 100, SCRW/2 + 200,  SCRH/2 + 100 , 20.0, 20.0, tan);
					al_draw_textf(font,brown,220.0,225.0,0,"%s",&distmessages[msg][0]);
					al_draw_textf(font,brown,220.0,300.0,0,"%s",distmessage2);
					glEnable(GL_DEPTH_TEST);
				}

				glMatrixMode(GL_MODELVIEW);
				glPopMatrix();
				glMatrixMode(GL_PROJECTION);
				glPopMatrix();
				glMatrixMode(GL_TEXTURE);
				glPopMatrix();
				glMatrixMode(GL_MODELVIEW);
				glEnable(GL_CULL_FACE);
				glEnable(GL_TEXTURE_2D);
				glEnable(GL_LIGHTING);
				al_flip_display();
				frames++;
				redraw--;
			}
		}
	}

done:

	printf("\n%.1f FPS\n", frames / (al_get_time() - start));
	printf("%d error\n",glGetError());
	al_destroy_event_queue(queue);
	al_destroy_display(display);
	al_uninstall_system();

	return 0;
}
