//#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <GL/gl.h>
#include "mersenne.h"

#define HALFWIDTH 0.5
#define QUARTERWIDTH 0.25

//FILE *fp;

static int sqrsize;
int terrainlist;

float *terrainfloats;

float getrand(void)
{
	return (genrand_real3() - 0.5) * (float)sqrsize;
}

int maketerrain(void)
{
	float *firstarray;
	float **firstrowptrs;
	float **terrainrowptrs;

	int width = 513;
	int halfwidth;
	int float513;
	int float256;
	int x,y,z;
	int endx,endy;
	int midx,midy;
	float radius;

	float513 = width * width;

	firstarray = malloc(sizeof(float) * float513);
	if(!firstarray)
	{
		return 1;
	}

	terrainfloats = malloc(sizeof(float) * float513);
	if(!terrainfloats)
	{
		free(firstarray);
		return 1;
	}

	firstrowptrs = malloc(sizeof(float *) * width);
	if(!firstrowptrs)
	{
		free(firstarray);
		free(terrainfloats);
		return 1;
	}

	for(x=0;x<width;x++)	//assign row pointers
	{
		y = x * width;
		firstrowptrs[x] = &firstarray[y];
	}

	terrainrowptrs = malloc(sizeof(float *) * width);
	if(!terrainrowptrs)
	{
		free(firstarray);
		free(terrainfloats);
		free(firstrowptrs);
		return 1;
	}

	for(x=0;x<width;x++)	//assign row pointers
	{
		y = x * width;
		terrainrowptrs[x] = &terrainfloats[y];
	}

	init_genrand(20);
	firstrowptrs[0][0] = getrand();
	firstrowptrs[width-1][0] = getrand();
	firstrowptrs[0][width-1] = getrand();
	firstrowptrs[width-1][width-1] = getrand();

	for(sqrsize=width-1;sqrsize>1;sqrsize/=2)
	{
		for(y=0;;y+=sqrsize)
		{
			endy = y + sqrsize;
			if(endy > width-1)
			{
				break;
			}
			for(x=0;;x+=sqrsize)
			{
				endx = x + sqrsize;
				if(endx > width-1)
				{
					break;
				}
				midx = x + (sqrsize/2);
				midy = y + (sqrsize/2);

				//calculate midpoint
				firstrowptrs[midx][midy] =
					(firstrowptrs[x][y] + firstrowptrs[endx][y] + firstrowptrs[x][endy] +
					firstrowptrs[endx][endy]) * QUARTERWIDTH + getrand();

				//top point of diamond
				firstrowptrs[midx][y] = (firstrowptrs[x][y] + firstrowptrs[endx][y]) * HALFWIDTH + getrand();

				//left point of diamond
				firstrowptrs[x][midy] = (firstrowptrs[x][y] + firstrowptrs[x][endy]) * HALFWIDTH + getrand();

				//right point of diamond
				firstrowptrs[endx][midy] = (firstrowptrs[endx][y] + firstrowptrs[endx][endy]) *
					HALFWIDTH + getrand();

				//bottom point of diamond
				firstrowptrs[midx][endy] = (firstrowptrs[x][endy] + firstrowptrs[endx][endy]) *
					HALFWIDTH + getrand();

				//new here, recalculate corner points from edge midpoints
				firstrowptrs[x][y]       = (firstrowptrs[midx][y] + firstrowptrs[x][midy]) * HALFWIDTH + getrand();
				firstrowptrs[endx][y]    = (firstrowptrs[midx][y] + firstrowptrs[endx][midy]) * HALFWIDTH + getrand();
				firstrowptrs[x][endy]    = (firstrowptrs[midx][endy] + firstrowptrs[x][midy]) * HALFWIDTH + getrand();
				firstrowptrs[endx][endy] = (firstrowptrs[midx][endy] + firstrowptrs[endx][midy]) * HALFWIDTH + getrand();
			}
		}
	}

	//copy your shiny new heightfield to a safe place
	for(x=0;x<float513;x++)
	{
		terrainfloats[x] = firstarray[x];
	}

	//the "top" side is a bit too low for a good crater
	y = width >> 1;
	for(x=0;x<y;x++)
	{
		for(z=0;z<width;z++)
		{
			float hgt = y - x;
			hgt *= 0.1;
			firstrowptrs[x][z] += hgt;
		}
	}

	//reduce the heights for genrand(2) a bit
	for(x=0;x<float513;x++)
	{
		firstarray[x] *= 0.4;
	}


	//now lower terrainfloats and sink it down a bit
	for(x=0;x<float513;x++)
	{
		terrainfloats[x] *= 0.1;
		//terrainfloats[x] += 40.0;
	}

	y = float513 >> 2;	// 1/4 of the width/length

	for(x=0;x<y;x++)
	{
		firstarray[x] = -500.0;	//make front end of entire firstarray well below the rest
	}

	x = y * 3;		//start at 3/4 mark to end

	for(;x<float513;x++)
	{
		firstarray[x] = -500.0;
	}

	y = width >> 2;
	for(x=0;x<y;x++)	//now do left side (looking down from above)
	{
		for(z=0;z<width;z++)
		{
			firstrowptrs[z][x] = -500.0;
		}
	}

	for(x=y*3;x<width;x++)	//now do right side
	{
		for(z=0;z<width;z++)
		{
			firstrowptrs[z][x] = -500.0;
		}
	}

	//now dish in the center part like a crater
	radius = (float)width/5.0;
	for(x=0;x<width;x++)
	{
		int halfx;
		halfx = x - width/2;
		for(y=0;y<width;y++)
		{
			int halfy;
			float dist;
			halfy = y - width/2;
			dist = sqrt(halfx*halfx + halfy*halfy)/2.0;
			if(dist < radius)
			{
				firstrowptrs[x+30][y-30] += dist-radius;
			}
		}
	}

	for(x=0;x<float513;x++)	//"or" the firstarrays together
	{
		if(firstarray[x] < terrainfloats[x])
		{
			firstarray[x] = terrainfloats[x];
		}
	}

	//now average the fuck out of it
	for(z=0;z<10;z++)
	{
		for(x=1;x<width-1;x++)
		{
			for(y=1;y<width-1;y++)
			{
				float avg;
				avg = firstrowptrs[x-1][y-1];
				avg += firstrowptrs[x][y-1];
				avg += firstrowptrs[x+1][y-1];
				avg += firstrowptrs[x-1][y];
				avg += firstrowptrs[x][y];
				avg += firstrowptrs[x+1][y];
				avg += firstrowptrs[x-1][y+1];
				avg += firstrowptrs[x][y+1];
				avg += firstrowptrs[x+1][y+1];
				avg /= 9.0;
				terrainrowptrs[x][y] = avg;
			}
		}
		for(x=0;x<float513;x++)
		{
			firstarray[x] = terrainfloats[x];
		}
	}

	free(terrainfloats);
	free(terrainrowptrs);

	halfwidth = width >> 1;
	float256 = halfwidth * halfwidth;

	terrainfloats = malloc(sizeof(float) * float256);
	if(!terrainfloats)
	{
		free(firstarray);
		free(firstrowptrs);
		return 1;
	}
	terrainrowptrs = malloc(sizeof(float *) * halfwidth);
	if(!terrainrowptrs)
	{
		free(firstarray);
		free(firstrowptrs);
		free(terrainfloats);
		return 1;
	}

	for(x=0;x<halfwidth;x++)	//assign row pointers
	{
		y = x * halfwidth;
		terrainrowptrs[x] = &terrainfloats[y];
	}

	//transfer the middle part
	z = halfwidth >> 1;
	for(x=0;x<halfwidth;x++)
	{
		for(y=0;y<halfwidth;y++)
		{
			terrainrowptrs[x][y] = firstrowptrs[x+z][y+z] * 15.0;	//the 15.0 should be 10.0 (?) since I told L3DT there were 10 meters per square, but this looks better
		}
	}

	/*	write out a file for L3DT to work on
	fp = fopen("test.hgt","w");
	fwrite(terrainfloats,sizeof(float),float256,fp);
	fclose(fp);
	*/

	/*  kludge to eliminate weird artifacts at the edges seen after making terrain
	for(x=0;x<256;x++)
	{
		//terrainfloats[x] = -1000.0;
		terrainfloats[256*256-x] = -500.0;
	} */
	for(x=0;x<256*255;x+=256)
	{
		for(y=0;y<12;y++)
		{
			terrainfloats[x+y] = -500.0;
		}
	}

	free(firstarray);
	free(terrainrowptrs);
	free(firstrowptrs);
	//return 0;
	width = 256;
	terrainlist = glGenLists(1);

	//Now put it all in a list
	glNewList(terrainlist,GL_COMPILE);
		glBegin(GL_TRIANGLES);
		{
			//glBegin doesn't need a scope, but it's handy to declare some floats here :)
			float *tx;
			float fx;
			float fx2;
			float fy;
			float fy2;
			float vx;
			float vy;
			float vx2;
			float vy2;
			float reciprocal = 1.0/256.0;
			fy2 = reciprocal;
			tx = terrainfloats;
			vy = 0.0;
			vy2 = 10.0;
			fy = 0.0;
			fy2 = reciprocal;
			for(y=0;y<width;y++)
			{
				//printf("x %d\n",x);
				vx = 0.0;
				vx2 = 10.0;
				fx = 0.0;
				fx2 = reciprocal;
				for(x=0;x<width;x++)
				{
					//upper left triangle of a quad

					//if(frames & 0x80)
					{
					glTexCoord2f(fx, fy);		//upper left vertex
					glVertex3f(vx, tx[0], vy);

					glTexCoord2f(fx, fy2);		//lower left vertex
					glVertex3f(vx, tx[width], vy2);

					glTexCoord2f(fx2, fy);		//upper right vertex
					glVertex3f(vx2, tx[1], vy);
					}
					//else
					{

					//lower right triangle of a quad

					glTexCoord2f(fx2, fy);		//upper right vertex
					glVertex3f(vx2, tx[1], vy);

					glTexCoord2f(fx, fy2);		//lower left vertex
					glVertex3f(vx, tx[width], vy2);

					glTexCoord2f(fx2, fy2);		//lower right vertex
					glVertex3f(vx2, tx[width+1], vy2);
					}
					//if( (frames == 1) || (frames == 2) )
					//	printf("%f %f %f\n",fx,fx2,reciprocal);
					fx = fx2;
					fx2 += reciprocal;
					vx = vx2;
					vx2 += 10.0;
					tx++;
				}
				//tx++;
				//printf("%p %p\n",tx ,&terrainfloats[y * width]);
				//printf("error\n");
				//printf("%f %f %f\n",fy,fy2,reciprocal);
				fy = fy2;
				fy2 += reciprocal;
				vy = vy2;
				vy2 += 10.0;
			}
		}
		glEnd();
	glEndList();

	return 0;
}
