#include <math.h>
#include "santamath.h"

/* you _can_ store result into source if you want */
/* out winds up being 1 unit (feet, meters, miles, but ONE) unit long */
void normalize_vector(POINT3D *in, POINT3D *out)
{
	float i = 1.0/(sqrt(in->x*in->x+in->y*in->y+in->z*in->z));
	out->x = in->x * i;
	out->y = in->y * i;
	out->z = in->z * i;
}

/* you have to count 0 as third leg, i.e. in1 & in2 have to be _vectors_! */
/* not just 3d points! */
void origin_cross_product(POINT3D *in1, POINT3D *in2, POINT3D *out)
{
	out->x = in1->y * in2->z - in1->z * in2->y;
	out->y = in1->z * in2->x - in1->x * in2->z;
	out->z = in1->x * in2->y - in1->y * in2->x;
}

/* this one can handle three points anywhere (triangle) without having */
/* to subtract elsewhere (vectorizing). Result is a vector! */
/* i.e. with respect to origin */
void inplace_cross_product(POINT3D *in1, POINT3D *in2, POINT3D *in3,
                           POINT3D *out)
{
	float px,py,pz;
	float qx,qy,qz;
	px = in1->x - in2->x;
	py = in1->y - in2->y;
	pz = in1->z - in2->z;
	qx = in3->x - in2->x;
	qy = in3->y - in2->y;
	qz = in3->z - in2->z;
	out->x = py * qz - pz * qy;
	out->y = pz * qx - px * qz;
	out->z = px * qy - py * qx;
}

float skidot_product(POINT3D *point1, POINT3D *point2)
/* points in same direction? (dist doesn't matter) = 1.000, points opposite */
/* direction? -1.000  90 degree angle? 0.0 and all in between. */
{
	return (point1->x*point2->x+point1->y*point2->y+point1->z*point2->z);
}

/*  Old and wrong (can bomb out w/divide by zero)
float get_dist_point2plane(PLANE *plane, POINT3D *p)
{
 return (plane->a*p->x+plane->b*p->y+plane->c*p->z+plane->d)/
         sqrt(plane->a*plane->a+plane->b*plane->b+plane->c*plane->c);
}
*/
/* plane vector (a,b,c) must be normalized! */
float get_dist_point2plane(PLANE *plane, POINT3D *p)
{
	return plane->a * p->x + plane->b * p->y + plane->c * p->z + plane->d ;
}

void get_plane_eq(POINT3D *p1, POINT3D *p2, POINT3D *p3,PLANE *plane)
{
	float px,py,pz;
	float qx,qy,qz;
	px = p1->x - p2->x;
	py = p1->y - p2->y;
	pz = p1->z - p2->z;
	qx = p3->x - p2->x;
	qy = p3->y - p2->y;
	qz = p3->z - p2->z;
	plane->a = py * qz - pz * qy;
	plane->b = pz * qx - px * qz;
	plane->c = px * qy - py * qx;
	plane->d = -(plane->a*p2->x+plane->b*p2->y+plane->c*p2->z);

	/* get unit normal while we're at it */
	qx = 1.0/(sqrt(plane->a*plane->a+plane->b*plane->b+plane->c*plane->c));
	plane->nx = plane->a * qx;
	plane->ny = plane->b * qx;
	plane->nz = plane->c * qx;
}

#define GRIDADJ 0.1
#define TERRAINWIDTH 256

extern float *terrainfloats;

float get_ground_height(POINT3D *contact)
{
	PLANE plane;

	POINT3D tri1,tri2,tri3;

	POINT3D sizeadj;
	POINT3D truncated;
	POINT3D remainder;

	int indx,indz;

	sizeadj.x = contact->x * -GRIDADJ;  //squash 'em down to heightfield poly size
	sizeadj.z = contact->z * -GRIDADJ;

	truncated.x = floor(sizeadj.x);       //integral parts
	truncated.z = floor(sizeadj.z);

	remainder.x = sizeadj.x - truncated.x;
	remainder.z = sizeadj.z - truncated.z;

	indx = truncated.x;
	indz = truncated.z;

	truncated.y = terrainfloats[indz*TERRAINWIDTH+indx];

	tri1.x = truncated.x;
	tri1.z = truncated.z;
	tri1.y = truncated.y;

	//decide if you're in the top right triangle or the bottom left tri
	if(remainder.x > remainder.z)  //yes
	{
		//v1 is next vertex for clockwise
		tri2.x = truncated.x + 1.0;
		tri2.z = truncated.z + 1.0;
		tri2.y = terrainfloats[ (indz+1) * TERRAINWIDTH + indx + 1];
		//v2
		tri3.x = tri2.x;
		tri3.z = truncated.z;
		tri3.y = terrainfloats[ indz * TERRAINWIDTH + indx + 1];
	}
	else //point is in the bottom left tri
	{
		//v3 is next vertex for clockwise
		tri2.x = truncated.x;
		tri2.z = truncated.z + 1.0;
		tri2.y = terrainfloats[ (indz+1) * TERRAINWIDTH + indx];
		//v2
		tri3.x = truncated.x + 1.0;
		tri3.z = tri1.z;
		tri3.y = terrainfloats[ indz * TERRAINWIDTH + indx + 1];
	}

	get_plane_eq(&tri1,&tri2,&tri3,&plane);

	//get y value for the x,z on this plane
	return (plane.a * sizeadj.x + plane.c * sizeadj.z + plane.d)/plane.b;
}

