#ifndef _SANTAMATH_H_
#define _SANTAMATH_H_

#define RADIANS_TO_DEGREES(radians) ((radians) * (180.0 / M_PI))
#define DEGREES_TO_RADIANS(angle) ((angle) / 180.0 * M_PI)

typedef struct
{
	float x;
	float y;
}POINT2D;

typedef struct
{
	float x;
	float y;
	float z;
}POINT3D;

typedef struct
{
	float a;
	float b;
	float c;
	float d;

	float nx;
	float ny;  //unit normal
	float nz;
}PLANE;

/* you _can_ store result into source if you want */
/* extern void normalize_vector(POINT3D *in, POINT3D *out); */
/*  */

/* you have to count 0 as third leg, i.e. in1 & in2 have to be _vectors_! */
/* not just 3d points! */
extern void origin_cross_product(POINT3D *in1, POINT3D *in2, POINT3D *out);

/* this one can handle three points anywhere (triangle) without having */
/* to subtract elsewhere (vectorizing). Result is a vector! */
/* i.e. with respect to origin */
extern void inplace_cross_product(POINT3D *in1, POINT3D *in2, POINT3D *in3,
                           POINT3D *out);

extern float skidot_product(POINT3D *point1, POINT3D *point2);

/* stores values in plane.a through plane.d as well as unit normal into */
/* plane.nx, ny, nz. */
extern void get_plane_eq(POINT3D *a, POINT3D *b, POINT3D *c, PLANE *plane);

/* if result is negative, it's distance *behind* plane (clockwise order) */
extern float get_dist_point2plane(PLANE *plane, POINT3D *p);

extern float get_ground_height(POINT3D *contact);

#endif /* _SANTAMATH_H_ */
