
/* Procedurally gnerate various sized spheres and spheroids to create body parts.
 * Should be much less than equivalent mesh files from Blender et. al. to keep under the 2Mb limit.
 */

#include <stdlib.h>	//for exit()
#include <allegro5/allegro_native_dialog.h>
#include "spheroid.h"

GLuint unitsphere;

//extern int texID;

POINT3D facepts[NUMLATS + 1][NUMLONGS + 1];

POINT2D texcoords[NUMLATS + 1][NUMLONGS + 1];

void genpts(void)
{
	POINT2D *sines;
	int i,j;
	float accum = 0.0;
	double lats = NUMLATS;
	lats = 1.0/lats;
	lats *= M_PI;

	sines = malloc(sizeof(POINT2D) * (NUMLONGS + 1));
	if(!sines)
	{
		al_show_native_message_box(0, "Error", "Out of memory", "", NULL, 0);
		exit(1);
	}

	for(i=0;i<NUMLONGS + 1;i++)
	{
		sines[i].x = sin(accum);
		sines[i].y = cos(accum);
		accum += lats;
	}

	for(i=0;i<NUMLATS + 1;i++)
	{
		for(j=0;j<NUMLONGS+1;j++)
		{
			facepts[i][j].x = sines[j].y * sines[i].x;
			facepts[i][j].y = sines[i].y;
			facepts[i][j].z = sines[j].x * sines[i].x;

			texcoords[i][j].x = (float)(i)/(float)(NUMLATS);
			texcoords[i][j].y = (float)(j)/(float)NUMLONGS;
			//printf("x % 2.4f y % 2.4f ",texcoords[i][j].x,texcoords[i][j].y);
		}
		//printf("\n");
	}

	free(sines);
	unitsphere = glGenLists(1);

	/*
	glNewList(unitsphere,GL_COMPILE);
		glBegin(GL_LINE_STRIP);
			for(i=0;i<NUMLATS + 1;i++)
			{
				for(j=0;j<NUMLONGS+1;j++)
				{
					glVertex3f(facepts[i][j].x, facepts[i][j].y, facepts[i][j].z);
				}
			}
		glEnd();
	glEndList();
	*/
	//glDisable(GL_TEXTURE_2D);

	glNewList(unitsphere,GL_COMPILE);
		glBegin(GL_TRIANGLES); //GL_LINE_STRIP); //GL_TRIANGLES works too
			for(i=0;i<NUMLATS;i++)
			{
				for(j=0;j<NUMLONGS;j++)
				{
					//upper left triangle of a quad
					glTexCoord2f(texcoords[i][j].x,texcoords[i][j].y);
					glNormal3f(facepts[i][j].x, facepts[i][j].y, facepts[i][j].z);		//upper left vertex
					glVertex3f(facepts[i][j].x, facepts[i][j].y, facepts[i][j].z);

					glTexCoord2f(texcoords[i+1][j].x,texcoords[i+1][j].y);
					glNormal3f(facepts[i+1][j].x, facepts[i+1][j].y, facepts[i+1][j].z);	//lower left vertex
					glVertex3f(facepts[i+1][j].x, facepts[i+1][j].y, facepts[i+1][j].z);

					glTexCoord2f(texcoords[i][j+1].x,texcoords[i][j+1].y);
					glNormal3f(facepts[i][j+1].x, facepts[i][j+1].y, facepts[i][j+1].z);	//upper right vertex
					glVertex3f(facepts[i][j+1].x, facepts[i][j+1].y, facepts[i][j+1].z);


					//lower right triangle of a quad
					glTexCoord2f(texcoords[i][j+1].x,texcoords[i][j+1].y);
					glNormal3f(facepts[i][j+1].x, facepts[i][j+1].y, facepts[i][j+1].z);	//upper right vertex
					glVertex3f(facepts[i][j+1].x, facepts[i][j+1].y, facepts[i][j+1].z);

					glTexCoord2f(texcoords[i+1][j].x,texcoords[i+1][j].y);
					glNormal3f(facepts[i+1][j].x, facepts[i+1][j].y, facepts[i+1][j].z);	//lower left vertex
					glVertex3f(facepts[i+1][j].x, facepts[i+1][j].y, facepts[i+1][j].z);

					glTexCoord2f(texcoords[i+1][j+1].x,texcoords[i+1][j+1].y);
					glNormal3f(facepts[i+1][j+1].x, facepts[i+1][j+1].y, facepts[i+1][j+1].z);	//lower right vertex
					glVertex3f(facepts[i+1][j+1].x, facepts[i+1][j+1].y, facepts[i+1][j+1].z);
				}
			}
		glEnd();
	glEndList();

}
